@@ -1,8 +1,6 @@
 #include "adw-demo-page-styles.h"
 
-#include <glib/gi18n.h>
-
-#include "adw-style-demo-window.h"
+#include "adw-style-demo-dialog.h"
 
 struct _AdwDemoPageStyles
 {
@@ -14,9 +12,7 @@ G_DEFINE_FINAL_TYPE (AdwDemoPageStyles, adw_demo_page_styles, ADW_TYPE_BIN)
 static void
 demo_run_cb (AdwDemoPageStyles *self)
 {
-  AdwStyleDemoWindow *window = adw_style_demo_window_new ();
-
-  adw_dialog_present (ADW_DIALOG (window), GTK_WIDGET (self));
+  adw_dialog_present (adw_style_demo_dialog_new (), GTK_WIDGET (self));
 }
 
 static void
