@@ -77,7 +77,6 @@ struct _AdwCarousel
   double position_shift;
 
   guint scroll_timeout_id;
-  gboolean can_scroll;
   gboolean is_being_allocated;
 };
 
@@ -344,6 +343,8 @@ animate_child_resize (AdwCarousel *self,
     adw_timed_animation_new (GTK_WIDGET (self), old_size,
                              value, duration, target);
 
+  adw_timed_animation_set_easing (ADW_TIMED_ANIMATION (child->resize_animation), ADW_EASE);
+
   g_signal_connect_swapped (child->resize_animation, "done",
                             G_CALLBACK (resize_animation_done_cb), child);
 
@@ -468,7 +469,7 @@ update_orientation (AdwCarousel *self)
 static void
 scroll_timeout_cb (AdwCarousel *self)
 {
-  self->can_scroll = TRUE;
+  self->scroll_timeout_id = 0;
 }
 
 static gboolean
@@ -487,7 +488,7 @@ scroll_cb (AdwCarousel              *self,
   if (!self->allow_scroll_wheel)
     return GDK_EVENT_PROPAGATE;
 
-  if (!self->can_scroll)
+  if (self->scroll_timeout_id > 0)
     return GDK_EVENT_PROPAGATE;
 
   if (!adw_carousel_get_interactive (self))
@@ -532,7 +533,6 @@ scroll_cb (AdwCarousel              *self,
 
   scroll_to (self, adw_carousel_get_nth_page (self, index), 0);
 
-  self->can_scroll = FALSE;
   self->scroll_timeout_id =
    g_timeout_add_once (SCROLL_TIMEOUT_DURATION,
                        (GSourceOnceFunc) scroll_timeout_cb,
@@ -883,7 +883,7 @@ adw_carousel_class_init (AdwCarouselClass *klass)
   widget_class->compute_expand = adw_widget_compute_expand;
 
   /**
-   * AdwCarousel:n-pages: (attributes org.gtk.Property.get=adw_carousel_get_n_pages)
+   * AdwCarousel:n-pages:
    *
    * The number of pages in a `AdwCarousel`.
    */
@@ -895,7 +895,7 @@ adw_carousel_class_init (AdwCarouselClass *klass)
                        G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
 
   /**
-   * AdwCarousel:position: (attributes org.gtk.Property.get=adw_carousel_get_position)
+   * AdwCarousel:position:
    *
    * Current scrolling position, unitless.
    *
@@ -909,7 +909,7 @@ adw_carousel_class_init (AdwCarouselClass *klass)
                          G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
 
   /**
-   * AdwCarousel:interactive: (attributes org.gtk.Property.get=adw_carousel_get_interactive org.gtk.Property.set=adw_carousel_set_interactive)
+   * AdwCarousel:interactive:
    *
    * Whether the carousel can be navigated.
    *
@@ -922,7 +922,7 @@ adw_carousel_class_init (AdwCarouselClass *klass)
                           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * AdwCarousel:spacing: (attributes org.gtk.Property.get=adw_carousel_get_spacing org.gtk.Property.set=adw_carousel_set_spacing)
+   * AdwCarousel:spacing:
    *
    * Spacing between pages in pixels.
    */
@@ -934,7 +934,7 @@ adw_carousel_class_init (AdwCarouselClass *klass)
                        G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * AdwCarousel:scroll-params: (attributes org.gtk.Property.get=adw_carousel_get_scroll_params org.gtk.Property.set=adw_carousel_set_scroll_params)
+   * AdwCarousel:scroll-params:
    *
    * Scroll animation spring parameters.
    *
@@ -950,7 +950,7 @@ adw_carousel_class_init (AdwCarouselClass *klass)
                         G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * AdwCarousel:allow-mouse-drag: (attributes org.gtk.Property.get=adw_carousel_get_allow_mouse_drag org.gtk.Property.set=adw_carousel_set_allow_mouse_drag)
+   * AdwCarousel:allow-mouse-drag:
    *
    * Sets whether the `AdwCarousel` can be dragged with mouse pointer.
    *
@@ -962,7 +962,7 @@ adw_carousel_class_init (AdwCarouselClass *klass)
                           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * AdwCarousel:allow-scroll-wheel: (attributes org.gtk.Property.get=adw_carousel_get_allow_scroll_wheel org.gtk.Property.set=adw_carousel_set_allow_scroll_wheel)
+   * AdwCarousel:allow-scroll-wheel:
    *
    * Whether the widget will respond to scroll wheel events.
    *
@@ -974,7 +974,7 @@ adw_carousel_class_init (AdwCarouselClass *klass)
                           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * AdwCarousel:allow-long-swipes: (attributes org.gtk.Property.get=adw_carousel_get_allow_long_swipes org.gtk.Property.set=adw_carousel_set_allow_long_swipes)
+   * AdwCarousel:allow-long-swipes:
    *
    * Whether to allow swiping for more than one page at a time.
    *
@@ -1047,7 +1047,6 @@ adw_carousel_init (AdwCarousel *self)
 
   self->orientation = GTK_ORIENTATION_HORIZONTAL;
   self->reveal_duration = 0;
-  self->can_scroll = TRUE;
 
   self->tracker = adw_swipe_tracker_new (ADW_SWIPEABLE (self));
   adw_swipe_tracker_set_allow_mouse_drag (self->tracker, TRUE);
@@ -1265,7 +1264,7 @@ adw_carousel_reorder (AdwCarousel *self,
                       GtkWidget   *child,
                       int          position)
 {
-  ChildInfo *info, *next_info;
+  ChildInfo *info, *next_info = NULL;
   GList *link, *next_link;
   int old_position, n_pages;
   double closest_point, old_point, new_point;
@@ -1465,7 +1464,7 @@ adw_carousel_get_nth_page (AdwCarousel *self,
 }
 
 /**
- * adw_carousel_get_n_pages: (attributes org.gtk.Method.get_property=n-pages)
+ * adw_carousel_get_n_pages:
  * @self: a carousel
  *
  * Gets the number of pages in @self.
@@ -1492,7 +1491,7 @@ adw_carousel_get_n_pages (AdwCarousel *self)
 }
 
 /**
- * adw_carousel_get_position: (attributes org.gtk.Method.get_property=position)
+ * adw_carousel_get_position:
  * @self: a carousel
  *
  * Gets current scroll position in @self, unitless.
@@ -1510,7 +1509,7 @@ adw_carousel_get_position (AdwCarousel *self)
 }
 
 /**
- * adw_carousel_get_interactive: (attributes org.gtk.Method.get_property=interactive)
+ * adw_carousel_get_interactive:
  * @self: a carousel
  *
  * Gets whether @self can be navigated.
@@ -1526,7 +1525,7 @@ adw_carousel_get_interactive (AdwCarousel *self)
 }
 
 /**
- * adw_carousel_set_interactive: (attributes org.gtk.Method.set_property=interactive)
+ * adw_carousel_set_interactive:
  * @self: a carousel
  * @interactive: whether @self can be navigated
  *
@@ -1552,7 +1551,7 @@ adw_carousel_set_interactive (AdwCarousel *self,
 }
 
 /**
- * adw_carousel_get_spacing: (attributes org.gtk.Method.get_property=spacing)
+ * adw_carousel_get_spacing:
  * @self: a carousel
  *
  * Gets spacing between pages in pixels.
@@ -1568,7 +1567,7 @@ adw_carousel_get_spacing (AdwCarousel *self)
 }
 
 /**
- * adw_carousel_set_spacing: (attributes org.gtk.Method.set_property=spacing)
+ * adw_carousel_set_spacing:
  * @self: a carousel
  * @spacing: the new spacing value
  *
@@ -1590,7 +1589,7 @@ adw_carousel_set_spacing (AdwCarousel *self,
 }
 
 /**
- * adw_carousel_get_scroll_params: (attributes org.gtk.Method.get_property=scroll-params)
+ * adw_carousel_get_scroll_params:
  * @self: a carousel
  *
  * Gets the scroll animation spring parameters for @self.
@@ -1606,7 +1605,7 @@ adw_carousel_get_scroll_params (AdwCarousel *self)
 }
 
 /**
- * adw_carousel_set_scroll_params: (attributes org.gtk.Method.set_property=scroll-params)
+ * adw_carousel_set_scroll_params:
  * @self: a carousel
  * @params: the new parameters
  *
@@ -1634,7 +1633,7 @@ adw_carousel_set_scroll_params (AdwCarousel     *self,
 }
 
 /**
- * adw_carousel_get_allow_mouse_drag: (attributes org.gtk.Method.get_property=allow-mouse-drag)
+ * adw_carousel_get_allow_mouse_drag:
  * @self: a carousel
  *
  * Sets whether @self can be dragged with mouse pointer.
@@ -1650,7 +1649,7 @@ adw_carousel_get_allow_mouse_drag (AdwCarousel *self)
 }
 
 /**
- * adw_carousel_set_allow_mouse_drag: (attributes org.gtk.Method.set_property=allow-mouse-drag)
+ * adw_carousel_set_allow_mouse_drag:
  * @self: a carousel
  * @allow_mouse_drag: whether @self can be dragged with mouse pointer
  *
@@ -1675,7 +1674,7 @@ adw_carousel_set_allow_mouse_drag (AdwCarousel *self,
 }
 
 /**
- * adw_carousel_get_allow_scroll_wheel: (attributes org.gtk.Method.get_property=allow-scroll-wheel)
+ * adw_carousel_get_allow_scroll_wheel:
  * @self: a carousel
  *
  * Gets whether @self will respond to scroll wheel events.
@@ -1691,7 +1690,7 @@ adw_carousel_get_allow_scroll_wheel (AdwCarousel *self)
 }
 
 /**
- * adw_carousel_set_allow_scroll_wheel: (attributes org.gtk.Method.set_property=allow-scroll-wheel)
+ * adw_carousel_set_allow_scroll_wheel:
  * @self: a carousel
  * @allow_scroll_wheel: whether @self will respond to scroll wheel events
  *
@@ -1716,7 +1715,7 @@ adw_carousel_set_allow_scroll_wheel (AdwCarousel *self,
 }
 
 /**
- * adw_carousel_get_allow_long_swipes: (attributes org.gtk.Method.get_property=allow-long-swipes)
+ * adw_carousel_get_allow_long_swipes:
  * @self: a carousel
  *
  * Gets whether to allow swiping for more than one page at a time.
@@ -1732,7 +1731,7 @@ adw_carousel_get_allow_long_swipes (AdwCarousel *self)
 }
 
 /**
- * adw_carousel_set_allow_long_swipes: (attributes org.gtk.Method.set_property=allow-long-swipes)
+ * adw_carousel_set_allow_long_swipes:
  * @self: a carousel
  * @allow_long_swipes: whether to allow long swipes
  *
@@ -1758,7 +1757,7 @@ adw_carousel_set_allow_long_swipes (AdwCarousel *self,
 }
 
 /**
- * adw_carousel_get_reveal_duration: (attributes org.gtk.Method.get_property=reveal-duration)
+ * adw_carousel_get_reveal_duration:
  * @self: a carousel
  *
  * Gets the page reveal duration, in milliseconds.
@@ -1774,7 +1773,7 @@ adw_carousel_get_reveal_duration (AdwCarousel *self)
 }
 
 /**
- * adw_carousel_set_reveal_duration: (attributes org.gtk.Method.set_property=reveal-duration)
+ * adw_carousel_set_reveal_duration:
  * @self: a carousel
  * @reveal_duration: the new reveal duration value
  *
