@@ -279,6 +279,7 @@ struct _AdwNavigationView
   gboolean transition_cancel;
   double transition_progress;
   gboolean gesture_active;
+  AdwNavigationDirection swipe_direction;
 
   AdwShadowHelper *shadow_helper;
   AdwSwipeTracker *swipe_tracker;
@@ -837,8 +838,8 @@ switch_page (AdwNavigationView *self,
 
   g_object_notify_by_pspec (G_OBJECT (self), props[PROP_VISIBLE_PAGE]);
 
-  if (prev_page && adw_navigation_page_get_tag (prev_page) &&
-      page && adw_navigation_page_get_tag (page)) {
+  if ((prev_page && adw_navigation_page_get_tag (prev_page)) ||
+      (page && adw_navigation_page_get_tag (page))) {
     g_object_notify_by_pspec (G_OBJECT (self), props[PROP_VISIBLE_PAGE_TAG]);
   }
 }
@@ -1244,22 +1245,41 @@ static void
 prepare_cb (AdwSwipeTracker        *tracker,
             AdwNavigationDirection  direction,
             AdwNavigationView      *self)
+{
+  if (!adw_navigation_view_get_visible_page (self))
+    return;
+
+  self->swipe_direction = ADW_NAVIGATION_DIRECTION_BACK;
+}
+
+static void
+begin_swipe_cb (AdwSwipeTracker   *tracker,
+                AdwNavigationView *self)
 {
   AdwNavigationPage *visible_page = adw_navigation_view_get_visible_page (self);
   AdwNavigationPage *new_page;
   gboolean remove_on_pop = FALSE;
 
-  if (!visible_page)
+  if (self->swipe_direction < 0)
     return;
 
-  if (direction == ADW_NAVIGATION_DIRECTION_BACK) {
-    if (!adw_navigation_page_get_can_pop (visible_page))
+  if (!visible_page) {
+    self->swipe_direction = -1;
+    return;
+  }
+
+  if (self->swipe_direction == ADW_NAVIGATION_DIRECTION_BACK) {
+    if (!adw_navigation_page_get_can_pop (visible_page)) {
+      self->swipe_direction = -1;
       return;
+    }
 
     new_page = adw_navigation_view_get_previous_page (self, visible_page);
 
-    if (!new_page)
+    if (!new_page) {
+      self->swipe_direction = -1;
       return;
+    }
 
   } else {
     new_page = get_next_page (self);
@@ -1277,9 +1297,9 @@ prepare_cb (AdwSwipeTracker        *tracker,
   self->showing_page = new_page;
   self->hiding_page = g_object_ref (visible_page);
 
-  self->transition_pop = (direction == ADW_NAVIGATION_DIRECTION_BACK);
+  self->transition_pop = (self->swipe_direction == ADW_NAVIGATION_DIRECTION_BACK);
 
-  if (direction == ADW_NAVIGATION_DIRECTION_BACK) {
+  if (self->swipe_direction == ADW_NAVIGATION_DIRECTION_BACK) {
     g_object_ref (new_page);
   } else {
     if (remove_on_pop)
@@ -1304,6 +1324,8 @@ prepare_cb (AdwSwipeTracker        *tracker,
   gtk_widget_queue_resize (GTK_WIDGET (self));
 
   adw_swipe_tracker_set_upper_overshoot (self->swipe_tracker, TRUE);
+
+  self->swipe_direction = -1;
 }
 
 static void
@@ -1330,6 +1352,8 @@ end_swipe_cb (AdwSwipeTracker   *tracker,
 {
   gboolean animate;
 
+  self->swipe_direction = -1;
+
   if (!self->gesture_active)
     return;
 
@@ -2031,11 +2055,15 @@ adw_navigation_view_init (AdwNavigationView *self)
 
   g_signal_connect (self->swipe_tracker, "prepare",
                     G_CALLBACK (prepare_cb), self);
+  g_signal_connect (self->swipe_tracker, "begin-swipe",
+                    G_CALLBACK (begin_swipe_cb), self);
   g_signal_connect (self->swipe_tracker, "update-swipe",
                     G_CALLBACK (update_swipe_cb), self);
   g_signal_connect (self->swipe_tracker, "end-swipe",
                     G_CALLBACK (end_swipe_cb), self);
 
+  self->swipe_direction = -1;
+
   self->shield = adw_gizmo_new ("widget", NULL, NULL, NULL, NULL, NULL, NULL);
   gtk_widget_set_child_visible (self->shield, FALSE);
   gtk_widget_set_parent (self->shield, GTK_WIDGET (self));
@@ -2864,7 +2892,7 @@ adw_navigation_view_replace (AdwNavigationView  *self,
   AdwNavigationPage *visible_page;
   GHashTable *added_pages;
   guint i, old_length;
-  gboolean had_visible_page, old_visible_page_has_tag = FALSE;
+  gboolean had_visible_page, old_visible_page_had_tag = FALSE;
 
   g_return_if_fail (ADW_IS_NAVIGATION_VIEW (self));
   g_return_if_fail (n_pages >= 0);
@@ -2891,7 +2919,7 @@ adw_navigation_view_replace (AdwNavigationView  *self,
     if (get_remove_on_pop (c) &&
         !g_hash_table_contains (added_pages, c)) {
       if (c == visible_page) {
-        old_visible_page_has_tag = adw_navigation_page_get_tag (visible_page) != NULL;
+        old_visible_page_had_tag = adw_navigation_page_get_tag (visible_page) != NULL;
         adw_navigation_page_hiding (visible_page);
         adw_navigation_page_hidden (visible_page);
         visible_page = NULL;
@@ -2936,7 +2964,7 @@ adw_navigation_view_replace (AdwNavigationView  *self,
   } else if (had_visible_page) {
     g_object_notify_by_pspec (G_OBJECT (self), props[PROP_VISIBLE_PAGE]);
 
-    if (old_visible_page_has_tag)
+    if (old_visible_page_had_tag)
       g_object_notify_by_pspec (G_OBJECT (self), props[PROP_VISIBLE_PAGE_TAG]);
   }
 
