@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2019 Alexander Mikhaylenko <exalm7659@gmail.com>
+ * Copyright (C) 2019 Alice Mikhaylenko <alicem@gnome.org>
  *
  * SPDX-License-Identifier: LGPL-2.1-or-later
  */
@@ -28,13 +28,15 @@
 #define ANIMATION_BASE_VELOCITY 0.002
 #define DRAG_THRESHOLD_DISTANCE 16
 #define EPSILON 0.005
+#define OVERSHOOT_DISTANCE_MULTIPLIER 0.1
 
 #define SIGN(x) ((x) > 0.0 ? 1.0 : ((x) < 0.0 ? -1.0 : 0.0))
 
 /**
  * AdwSwipeTracker:
  *
- * A swipe tracker used in [class@Carousel], [class@Flap] and [class@Leaflet].
+ * A swipe tracker used in [class@Carousel], [class@NavigationView] and
+ * [class@OverlaySplitView].
  *
  * The `AdwSwipeTracker` object can be used for implementing widgets with swipe
  * gestures. It supports touch-based swipes, pointer dragging, and touchpad
@@ -69,6 +71,9 @@ struct _AdwSwipeTracker
   gboolean allow_mouse_drag;
   gboolean allow_long_swipes;
   GtkOrientation orientation;
+  gboolean lower_overshoot;
+  gboolean upper_overshoot;
+  gboolean allow_window_handle;
 
   double pointer_x;
   double pointer_y;
@@ -87,6 +92,9 @@ struct _AdwSwipeTracker
   GtkEventController *scroll_controller;
   GtkGesture *touch_gesture;
   GtkGesture *touch_gesture_capture;
+
+  gboolean is_window_handle;
+  gboolean ignore_direction;
 };
 
 G_DEFINE_FINAL_TYPE_WITH_CODE (AdwSwipeTracker, adw_swipe_tracker, G_TYPE_OBJECT,
@@ -99,10 +107,13 @@ enum {
   PROP_REVERSED,
   PROP_ALLOW_MOUSE_DRAG,
   PROP_ALLOW_LONG_SWIPES,
+  PROP_LOWER_OVERSHOOT,
+  PROP_UPPER_OVERSHOOT,
+  PROP_ALLOW_WINDOW_HANDLE,
 
   /* GtkOrientable */
   PROP_ORIENTATION,
-  LAST_PROP = PROP_ALLOW_LONG_SWIPES + 1,
+  LAST_PROP = PROP_ALLOW_WINDOW_HANDLE + 1,
 };
 
 static GParamSpec *props[LAST_PROP];
@@ -192,76 +203,6 @@ gesture_prepare (AdwSwipeTracker        *self,
   self->state = ADW_SWIPE_TRACKER_STATE_PENDING;
 }
 
-static void
-trim_history (AdwSwipeTracker *self,
-              guint32          current_time)
-{
-  guint32 threshold_time = current_time - EVENT_HISTORY_THRESHOLD_MS;
-  guint i;
-
-  for (i = 0; i < self->event_history->len; i++) {
-    guint32 time = g_array_index (self->event_history,
-                                  EventHistoryRecord, i).time;
-
-    if (time >= threshold_time)
-      break;
-  }
-
-  if (i > 0)
-    g_array_remove_range (self->event_history, 0, i);
-}
-
-static void
-append_to_history (AdwSwipeTracker *self,
-                   double           delta,
-                   guint32          time)
-{
-  EventHistoryRecord record;
-
-  trim_history (self, time);
-
-  record.delta = delta;
-  record.time = time;
-
-  g_array_append_val (self->event_history, record);
-}
-
-static double
-calculate_velocity (AdwSwipeTracker *self)
-{
-  double total_delta = 0;
-  guint32 first_time = 0, last_time = 0;
-  guint i;
-
-  for (i = 0; i < self->event_history->len; i++) {
-    EventHistoryRecord *r =
-      &g_array_index (self->event_history, EventHistoryRecord, i);
-
-    if (i == 0)
-      first_time = r->time;
-    else
-      total_delta += r->delta;
-
-    last_time = r->time;
-  }
-
-  if (first_time == last_time)
-    return 0;
-
-  return total_delta / (last_time - first_time);
-}
-
-static void
-gesture_begin (AdwSwipeTracker *self)
-{
-  if (self->state != ADW_SWIPE_TRACKER_STATE_PENDING)
-    return;
-
-  self->state = ADW_SWIPE_TRACKER_STATE_SCROLLING;
-
-  g_signal_emit (self, signals[SIGNAL_BEGIN_SWIPE], 0);
-}
-
 static int
 find_closest_point (double *points,
                     int     n,
@@ -304,23 +245,6 @@ find_previous_point (double *points,
   return -1;
 }
 
-static int
-find_point_for_projection (AdwSwipeTracker *self,
-                           double          *points,
-                           int              n,
-                           double           pos,
-                           double           velocity)
-{
-  int initial = find_closest_point (points, n, self->initial_progress);
-  int prev = find_previous_point (points, n, pos);
-  int next = find_next_point (points, n, pos);
-
-  if ((velocity > 0 ? prev : next) == initial)
-    return velocity > 0 ? next : prev;
-
-  return find_closest_point (points, n, pos);
-}
-
 static void
 get_bounds (AdwSwipeTracker *self,
             double          *points,
@@ -343,6 +267,129 @@ get_bounds (AdwSwipeTracker *self,
   *upper = points[MIN (next + 1, n - 1)];
 }
 
+static double
+adjust_for_overshoot (AdwSwipeTracker *self,
+                      double           amount)
+{
+  double d = adw_swipeable_get_distance (self->swipeable) * OVERSHOOT_DISTANCE_MULTIPLIER;
+
+  return (1 - 1 / (1 + amount * d)) / d;
+}
+
+static void
+trim_history (AdwSwipeTracker *self,
+              guint32          current_time)
+{
+  guint32 threshold_time = current_time - EVENT_HISTORY_THRESHOLD_MS;
+  guint i;
+
+  for (i = 0; i < self->event_history->len; i++) {
+    guint32 time = g_array_index (self->event_history,
+                                  EventHistoryRecord, i).time;
+
+    if (time >= threshold_time)
+      break;
+  }
+
+  if (i > 0)
+    g_array_remove_range (self->event_history, 0, i);
+}
+
+static void
+append_to_history (AdwSwipeTracker *self,
+                   double           delta,
+                   guint32          time)
+{
+  EventHistoryRecord record;
+
+  trim_history (self, time);
+
+  record.delta = delta;
+  record.time = time;
+
+  g_array_append_val (self->event_history, record);
+}
+
+static double
+calculate_velocity (AdwSwipeTracker *self)
+{
+  double total_delta = 0, velocity, lower, upper;
+  double *points;
+  guint32 first_time = 0, last_time = 0;
+  guint i;
+  int n;
+
+  for (i = 0; i < self->event_history->len; i++) {
+    EventHistoryRecord *r =
+      &g_array_index (self->event_history, EventHistoryRecord, i);
+
+    if (i == 0)
+      first_time = r->time;
+    else
+      total_delta += r->delta;
+
+    last_time = r->time;
+  }
+
+  if (first_time == last_time)
+    return 0;
+
+  velocity = total_delta / (last_time - first_time);
+
+  /* Overshoot */
+
+  points = adw_swipeable_get_snap_points (self->swipeable, &n);
+
+  if (!self->allow_long_swipes)
+    get_bounds (self, points, n, self->initial_progress, &lower, &upper);
+  else
+    get_range (self, &lower, &upper);
+
+  if (self->progress <= lower) {
+    if (self->lower_overshoot && self->progress > lower)
+      velocity *= adjust_for_overshoot (self, lower - self->progress) / (lower - self->progress);
+    else if (velocity < 0)
+      velocity = 0;
+  }
+
+  if (self->progress >= upper) {
+    if (self->upper_overshoot && self->progress < upper)
+      velocity *= adjust_for_overshoot (self, self->progress - upper) / (self->progress - upper);
+    else if (velocity > 0)
+      velocity = 0;
+  }
+
+  return velocity;
+}
+
+static void
+gesture_begin (AdwSwipeTracker *self)
+{
+  if (self->state != ADW_SWIPE_TRACKER_STATE_PENDING)
+    return;
+
+  self->state = ADW_SWIPE_TRACKER_STATE_SCROLLING;
+
+  g_signal_emit (self, signals[SIGNAL_BEGIN_SWIPE], 0);
+}
+
+static int
+find_point_for_projection (AdwSwipeTracker *self,
+                           double          *points,
+                           int              n,
+                           double           pos,
+                           double           velocity)
+{
+  int initial = find_closest_point (points, n, self->initial_progress);
+  int prev = find_previous_point (points, n, pos);
+  int next = find_next_point (points, n, pos);
+
+  if ((velocity > 0 ? prev : next) == initial)
+    return velocity > 0 ? next : prev;
+
+  return find_closest_point (points, n, pos);
+}
+
 static void
 gesture_update (AdwSwipeTracker *self,
                 double           delta,
@@ -367,10 +414,23 @@ gesture_update (AdwSwipeTracker *self,
   }
 
   progress = self->progress + delta;
-  progress = CLAMP (progress, lower, upper);
 
   self->progress = progress;
 
+  if (progress < lower) {
+    if (self->lower_overshoot)
+      progress = lower - adjust_for_overshoot (self, lower - progress);
+    else
+      progress = self->progress = lower;
+  }
+
+  if (progress > upper) {
+    if (self->upper_overshoot)
+      progress = upper + adjust_for_overshoot (self, progress - upper);
+    else
+      progress = self->progress = upper;
+  }
+
   g_signal_emit (self, signals[SIGNAL_UPDATE_SWIPE], 0, progress);
 }
 
@@ -389,8 +449,14 @@ get_end_progress (AdwSwipeTracker *self,
 
   points = adw_swipeable_get_snap_points (self->swipeable, &n);
 
+  if (!self->allow_long_swipes)
+    get_bounds (self, points, n, self->initial_progress, &lower, &upper);
+  else
+    get_range (self, &lower, &upper);
+
   if (ABS (velocity) < (is_touchpad ? VELOCITY_THRESHOLD_TOUCHPAD : VELOCITY_THRESHOLD_TOUCH)) {
     pos = points[find_closest_point (points, n, self->progress)];
+    pos = CLAMP (pos, lower, upper);
 
     g_free (points);
 
@@ -413,11 +479,6 @@ get_end_progress (AdwSwipeTracker *self,
 
   pos = (pos * SIGN (velocity)) + self->progress;
 
-  if (!self->allow_long_swipes)
-    get_bounds (self, points, n, self->initial_progress, &lower, &upper);
-  else
-    get_range (self, &lower, &upper);
-
   pos = CLAMP (pos, lower, upper);
   pos = points[find_point_for_projection (self, points, n, pos, velocity)];
 
@@ -468,8 +529,8 @@ gesture_cancel (AdwSwipeTracker *self,
 }
 
 static gboolean
-should_suppress_drag (AdwSwipeTracker *self,
-                      GtkWidget       *widget)
+has_window_handle (AdwSwipeTracker *self,
+                   GtkWidget       *widget)
 {
   GtkWidget *parent = widget;
   gboolean found_window_handle = FALSE;
@@ -483,6 +544,19 @@ should_suppress_drag (AdwSwipeTracker *self,
   return found_window_handle;
 }
 
+static gboolean
+should_suppress_drag (AdwSwipeTracker *self,
+                      GtkWidget       *widget)
+{
+  return !self->allow_window_handle && has_window_handle (self, widget);
+}
+
+static gboolean
+should_force_drag (AdwSwipeTracker *self,
+                   GtkWidget       *widget)
+{
+  return self->allow_window_handle && has_window_handle (self, widget);
+}
 
 static inline gboolean
 is_in_swipe_area (AdwSwipeTracker        *self,
@@ -495,7 +569,8 @@ is_in_swipe_area (AdwSwipeTracker        *self,
 
   adw_swipeable_get_swipe_area (self->swipeable, direction, is_drag, &rect);
 
-  return (G_APPROX_VALUE (x, rect.x, DBL_EPSILON) || x > rect.x) &&
+  return rect.width > 0 && rect.height > 0 &&
+         (G_APPROX_VALUE (x, rect.x, DBL_EPSILON) || x > rect.x) &&
          x < rect.x + rect.width &&
          (G_APPROX_VALUE (y, rect.y, DBL_EPSILON) || y > rect.y) &&
          y < rect.y + rect.height;
@@ -507,7 +582,25 @@ drag_capture_begin_cb (AdwSwipeTracker *self,
                        double           start_y,
                        GtkGestureDrag  *gesture)
 {
+  GtkWidget *widget;
+
+  if (self->state != ADW_SWIPE_TRACKER_STATE_NONE) {
+    gtk_gesture_set_state (GTK_GESTURE (gesture), GTK_EVENT_SEQUENCE_DENIED);
+    return;
+  }
+
+  widget = gtk_widget_pick (GTK_WIDGET (self->swipeable),
+                            start_x,
+                            start_y,
+                            GTK_PICK_DEFAULT);
+
+  if (should_force_drag (self, widget)) {
+    self->is_window_handle = TRUE;
+    return;
+  }
+
   gtk_gesture_set_state (GTK_GESTURE (gesture), GTK_EVENT_SEQUENCE_DENIED);
+  self->is_window_handle = FALSE;
 }
 
 static void
@@ -524,9 +617,14 @@ drag_begin_cb (AdwSwipeTracker *self,
   }
 
   widget = gtk_widget_pick (GTK_WIDGET (self->swipeable),
-                          start_x,
-                          start_y,
-                          GTK_PICK_DEFAULT);
+                            start_x,
+                            start_y,
+                            GTK_PICK_DEFAULT);
+
+  if (should_force_drag (self, widget))
+    return;
+
+  self->is_window_handle = FALSE;
 
   if (should_suppress_drag (self, widget)) {
     gtk_gesture_set_state (GTK_GESTURE (gesture), GTK_EVENT_SEQUENCE_DENIED);
@@ -569,45 +667,78 @@ drag_update_cb (AdwSwipeTracker *self,
   append_to_history (self, delta, time);
 
   if (self->state == ADW_SWIPE_TRACKER_STATE_NONE) {
-    if (is_vertical == is_offset_vertical)
-      gesture_prepare (self, offset > 0 ? ADW_NAVIGATION_DIRECTION_FORWARD : ADW_NAVIGATION_DIRECTION_BACK);
-    else
+    if (!self->ignore_direction && is_vertical != is_offset_vertical) {
       gtk_gesture_set_state (GTK_GESTURE (gesture), GTK_EVENT_SEQUENCE_DENIED);
-    return;
+      return;
+    }
+
+    gesture_prepare (self, offset > 0 ? ADW_NAVIGATION_DIRECTION_FORWARD : ADW_NAVIGATION_DIRECTION_BACK);
   }
 
   if (self->state == ADW_SWIPE_TRACKER_STATE_PENDING) {
-    double drag_distance;
+    double drag_distance, threshold;
     double first_point, last_point;
-    gboolean is_overshooting;
 
     get_range (self, &first_point, &last_point);
 
     drag_distance = sqrt (offset_x * offset_x + offset_y * offset_y);
-    is_overshooting = (offset < 0 && (G_APPROX_VALUE (self->progress, first_point, DBL_EPSILON) ||
-                                      self->progress < first_point)) ||
-                      (offset > 0 && (G_APPROX_VALUE (self->progress, last_point, DBL_EPSILON) ||
-                                      self->progress > last_point));
 
-    if (G_APPROX_VALUE (drag_distance, DRAG_THRESHOLD_DISTANCE, DBL_EPSILON) ||
-        drag_distance > DRAG_THRESHOLD_DISTANCE) {
+    if (self->is_window_handle) {
+      GtkSettings *settings = gtk_widget_get_settings (GTK_WIDGET (self->swipeable));
+      int dnd_threshold;
+
+      g_object_get (settings, "gtk-dnd-drag-threshold", &dnd_threshold, NULL);
+
+      threshold = dnd_threshold;
+    } else {
+      threshold = DRAG_THRESHOLD_DISTANCE;
+    }
+
+    if (G_APPROX_VALUE (drag_distance, threshold, DBL_EPSILON) ||
+        drag_distance > threshold) {
       double start_x, start_y;
       AdwNavigationDirection direction;
+      gboolean is_overshooting_lower, is_overshooting_upper;
 
       gtk_gesture_drag_get_start_point (gesture, &start_x, &start_y);
       direction = offset > 0 ? ADW_NAVIGATION_DIRECTION_FORWARD : ADW_NAVIGATION_DIRECTION_BACK;
 
       if (!is_in_swipe_area (self, start_x, start_y, direction, TRUE) &&
-          !is_in_swipe_area (self, start_x + offset_x, start_y + offset_y, direction, TRUE))
-        return;
-
-      if ((is_vertical == is_offset_vertical) && !is_overshooting) {
-        gesture_begin (self);
-        self->prev_offset = offset;
-        gtk_gesture_set_state (GTK_GESTURE (gesture), GTK_EVENT_SEQUENCE_CLAIMED);
-      } else {
+          !is_in_swipe_area (self, start_x + offset_x, start_y + offset_y, direction, TRUE)) {
         gtk_gesture_set_state (GTK_GESTURE (gesture), GTK_EVENT_SEQUENCE_DENIED);
+        return;
       }
+
+      if (!self->ignore_direction && is_vertical != is_offset_vertical) {
+        gtk_gesture_set_state (GTK_GESTURE (gesture), GTK_EVENT_SEQUENCE_DENIED);
+        return;
+      }
+
+      if (G_APPROX_VALUE (first_point, last_point, DBL_EPSILON)) {
+        gtk_gesture_set_state (GTK_GESTURE (gesture), GTK_EVENT_SEQUENCE_DENIED);
+        return;
+      }
+
+      is_overshooting_lower =
+        offset < 0 &&
+        (G_APPROX_VALUE (self->progress, first_point, DBL_EPSILON) ||
+         self->progress < first_point);
+
+      is_overshooting_upper =
+        offset > 0 &&
+        (G_APPROX_VALUE (self->progress, last_point, DBL_EPSILON) ||
+         self->progress > last_point);
+
+      if ((!self->lower_overshoot && is_overshooting_lower) ||
+          (!self->upper_overshoot && is_overshooting_upper)) {
+        gtk_gesture_set_state (GTK_GESTURE (gesture), GTK_EVENT_SEQUENCE_DENIED);
+        return;
+      }
+
+      gesture_begin (self);
+
+      self->prev_offset = offset;
+      gtk_gesture_set_state (GTK_GESTURE (gesture), GTK_EVENT_SEQUENCE_CLAIMED);
     }
   }
 
@@ -642,6 +773,7 @@ drag_end_cb (AdwSwipeTracker *self,
   }
 
   gesture_end (self, distance, time, FALSE);
+  gtk_event_controller_reset (GTK_EVENT_CONTROLLER (self->touch_gesture));
 }
 
 static void
@@ -715,24 +847,31 @@ handle_scroll_event (AdwSwipeTracker *self,
   time = gdk_event_get_time (event);
 
   if (self->state == ADW_SWIPE_TRACKER_STATE_PENDING) {
-    gboolean is_overshooting;
     double first_point, last_point;
 
     get_range (self, &first_point, &last_point);
 
-    is_overshooting = (delta < 0 &&
-                       (G_APPROX_VALUE (self->progress, first_point, DBL_EPSILON) ||
-                        self->progress < first_point)) ||
-                      (delta > 0 &&
-                       (G_APPROX_VALUE (self->progress, last_point, DBL_EPSILON) ||
-                        self->progress > last_point));
-
     append_to_history (self, delta, time);
 
-    if (!is_overshooting)
-      gesture_begin (self);
-    else
+    if (G_APPROX_VALUE (first_point, last_point, DBL_EPSILON)) {
       gesture_cancel (self, distance, time, TRUE);
+    } else {
+      gboolean is_overshooting_lower =
+        delta < 0 &&
+        (G_APPROX_VALUE (self->progress, first_point, DBL_EPSILON) ||
+         self->progress < first_point);
+
+      gboolean is_overshooting_upper =
+        delta > 0 &&
+        (G_APPROX_VALUE (self->progress, last_point, DBL_EPSILON) ||
+         self->progress > last_point);
+
+      if ((!self->lower_overshoot && is_overshooting_lower) ||
+          (!self->upper_overshoot && is_overshooting_upper))
+        gesture_cancel (self, distance, time, TRUE);
+      else
+        gesture_begin (self);
+    }
   }
 
   if (self->state == ADW_SWIPE_TRACKER_STATE_SCROLLING) {
@@ -963,6 +1102,18 @@ adw_swipe_tracker_get_property (GObject    *object,
     g_value_set_boolean (value, adw_swipe_tracker_get_allow_long_swipes (self));
     break;
 
+  case PROP_LOWER_OVERSHOOT:
+    g_value_set_boolean (value, adw_swipe_tracker_get_lower_overshoot (self));
+    break;
+
+  case PROP_UPPER_OVERSHOOT:
+    g_value_set_boolean (value, adw_swipe_tracker_get_upper_overshoot (self));
+    break;
+
+  case PROP_ALLOW_WINDOW_HANDLE:
+    g_value_set_boolean (value, adw_swipe_tracker_get_allow_window_handle (self));
+    break;
+
   case PROP_ORIENTATION:
     g_value_set_enum (value, self->orientation);
     break;
@@ -1001,6 +1152,18 @@ adw_swipe_tracker_set_property (GObject      *object,
     adw_swipe_tracker_set_allow_long_swipes (self, g_value_get_boolean (value));
     break;
 
+  case PROP_LOWER_OVERSHOOT:
+    adw_swipe_tracker_set_lower_overshoot (self, g_value_get_boolean (value));
+    break;
+
+  case PROP_UPPER_OVERSHOOT:
+    adw_swipe_tracker_set_upper_overshoot (self, g_value_get_boolean (value));
+    break;
+
+  case PROP_ALLOW_WINDOW_HANDLE:
+    adw_swipe_tracker_set_allow_window_handle (self, g_value_get_boolean (value));
+    break;
+
   case PROP_ORIENTATION:
     set_orientation (self, g_value_get_enum (value));
     break;
@@ -1022,7 +1185,7 @@ adw_swipe_tracker_class_init (AdwSwipeTrackerClass *klass)
   object_class->set_property = adw_swipe_tracker_set_property;
 
   /**
-   * AdwSwipeTracker:swipeable: (attributes org.gtk.Property.get=adw_swipe_tracker_get_swipeable)
+   * AdwSwipeTracker:swipeable:
    *
    * The widget the swipe tracker is attached to.
    */
@@ -1032,7 +1195,7 @@ adw_swipe_tracker_class_init (AdwSwipeTrackerClass *klass)
                          G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY | G_PARAM_STATIC_STRINGS);
 
   /**
-   * AdwSwipeTracker:enabled: (attributes org.gtk.Property.get=adw_swipe_tracker_get_enabled org.gtk.Property.set=adw_swipe_tracker_set_enabled)
+   * AdwSwipeTracker:enabled:
    *
    * Whether the swipe tracker is enabled.
    *
@@ -1045,7 +1208,7 @@ adw_swipe_tracker_class_init (AdwSwipeTrackerClass *klass)
                           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * AdwSwipeTracker:reversed: (attributes org.gtk.Property.get=adw_swipe_tracker_get_reversed org.gtk.Property.set=adw_swipe_tracker_set_reversed)
+   * AdwSwipeTracker:reversed:
    *
    * Whether to reverse the swipe direction.
    *
@@ -1058,7 +1221,7 @@ adw_swipe_tracker_class_init (AdwSwipeTrackerClass *klass)
                           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * AdwSwipeTracker:allow-mouse-drag: (attributes org.gtk.Property.get=adw_swipe_tracker_get_allow_mouse_drag org.gtk.Property.set=adw_swipe_tracker_set_allow_mouse_drag)
+   * AdwSwipeTracker:allow-mouse-drag:
    *
    * Whether to allow dragging with mouse pointer.
    */
@@ -1068,7 +1231,7 @@ adw_swipe_tracker_class_init (AdwSwipeTrackerClass *klass)
                           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * AdwSwipeTracker:allow-long-swipes: (attributes org.gtk.Property.get=adw_swipe_tracker_get_allow_long_swipes org.gtk.Property.set=adw_swipe_tracker_set_allow_long_swipes)
+   * AdwSwipeTracker:allow-long-swipes:
    *
    * Whether to allow swiping for more than one snap point at a time.
    *
@@ -1080,6 +1243,44 @@ adw_swipe_tracker_class_init (AdwSwipeTrackerClass *klass)
                           FALSE,
                           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
+  /**
+   * AdwSwipeTracker:lower-overshoot:
+   *
+   * Whether to allow swiping past the first available snap point.
+   *
+   * Since: 1.4
+   */
+  props[PROP_LOWER_OVERSHOOT] =
+    g_param_spec_boolean ("lower-overshoot", NULL, NULL,
+                          FALSE,
+                          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
+
+  /**
+   * AdwSwipeTracker:upper-overshoot:
+   *
+   * Whether to allow swiping past the last available snap point.
+   *
+   * Since: 1.4
+   */
+  props[PROP_UPPER_OVERSHOOT] =
+    g_param_spec_boolean ("upper-overshoot", NULL, NULL,
+                          FALSE,
+                          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
+
+  /**
+   * AdwSwipeTracker:allow-window-handle:
+   *
+   * Whether to allow touchscreen swiping from `GtkWindowHandle`.
+   *
+   * This will make dragging the window impossible.
+   *
+   * Since: 1.5
+   */
+  props[PROP_ALLOW_WINDOW_HANDLE] =
+    g_param_spec_boolean ("allow-window-handle", NULL, NULL,
+                          FALSE,
+                          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
+
   g_object_class_override_property (object_class,
                                     PROP_ORIENTATION,
                                     "orientation");
@@ -1207,7 +1408,7 @@ adw_swipe_tracker_new (AdwSwipeable *swipeable)
 }
 
 /**
- * adw_swipe_tracker_get_swipeable: (attributes org.gtk.Method.get_property=swipeable)
+ * adw_swipe_tracker_get_swipeable:
  * @self: a swipe tracker
  *
  * Get the widget @self is attached to.
@@ -1223,7 +1424,7 @@ adw_swipe_tracker_get_swipeable (AdwSwipeTracker *self)
 }
 
 /**
- * adw_swipe_tracker_get_enabled: (attributes org.gtk.Method.get_property=enabled)
+ * adw_swipe_tracker_get_enabled:
  * @self: a swipe tracker
  *
  * Gets whether @self is enabled.
@@ -1239,7 +1440,7 @@ adw_swipe_tracker_get_enabled (AdwSwipeTracker *self)
 }
 
 /**
- * adw_swipe_tracker_set_enabled: (attributes org.gtk.Method.set_property=enabled)
+ * adw_swipe_tracker_set_enabled:
  * @self: a swipe tracker
  * @enabled: whether @self is enabled
  *
@@ -1270,7 +1471,7 @@ adw_swipe_tracker_set_enabled (AdwSwipeTracker *self,
 }
 
 /**
- * adw_swipe_tracker_get_reversed: (attributes org.gtk.Method.get_property=reversed)
+ * adw_swipe_tracker_get_reversed:
  * @self: a swipe tracker
  *
  * Gets whether @self is reversing the swipe direction.
@@ -1286,7 +1487,7 @@ adw_swipe_tracker_get_reversed (AdwSwipeTracker *self)
 }
 
 /**
- * adw_swipe_tracker_set_reversed: (attributes org.gtk.Method.set_property=reversed)
+ * adw_swipe_tracker_set_reversed:
  * @self: a swipe tracker
  * @reversed: whether to reverse the swipe direction
  *
@@ -1311,7 +1512,7 @@ adw_swipe_tracker_set_reversed (AdwSwipeTracker *self,
 }
 
 /**
- * adw_swipe_tracker_get_allow_mouse_drag: (attributes org.gtk.Method.get_property=allow-mouse-drag)
+ * adw_swipe_tracker_get_allow_mouse_drag:
  * @self: a swipe tracker
  *
  * Gets whether @self can be dragged with mouse pointer.
@@ -1327,7 +1528,7 @@ adw_swipe_tracker_get_allow_mouse_drag (AdwSwipeTracker *self)
 }
 
 /**
- * adw_swipe_tracker_set_allow_mouse_drag: (attributes org.gtk.Method.set_property=allow-mouse-drag)
+ * adw_swipe_tracker_set_allow_mouse_drag:
  * @self: a swipe tracker
  * @allow_mouse_drag: whether to allow mouse dragging
  *
@@ -1352,7 +1553,7 @@ adw_swipe_tracker_set_allow_mouse_drag (AdwSwipeTracker *self,
 }
 
 /**
- * adw_swipe_tracker_get_allow_long_swipes: (attributes org.gtk.Method.get_property=allow-long-swipes)
+ * adw_swipe_tracker_get_allow_long_swipes:
  * @self: a swipe tracker
  *
  * Gets whether to allow swiping for more than one snap point at a time.
@@ -1368,7 +1569,7 @@ adw_swipe_tracker_get_allow_long_swipes (AdwSwipeTracker *self)
 }
 
 /**
- * adw_swipe_tracker_set_allow_long_swipes: (attributes org.gtk.Method.set_property=allow-long-swipes)
+ * adw_swipe_tracker_set_allow_long_swipes:
  * @self: a swipe tracker
  * @allow_long_swipes: whether to allow long swipes
  *
@@ -1393,6 +1594,137 @@ adw_swipe_tracker_set_allow_long_swipes (AdwSwipeTracker *self,
   g_object_notify_by_pspec (G_OBJECT (self), props[PROP_ALLOW_LONG_SWIPES]);
 }
 
+/**
+ * adw_swipe_tracker_get_lower_overshoot:
+ * @self: a swipe tracker
+ *
+ * Gets whether to allow swiping past the first available snap point.
+ *
+ * Returns: whether to allow swiping past the first available snap point
+ *
+ * Since: 1.4
+ */
+gboolean
+adw_swipe_tracker_get_lower_overshoot (AdwSwipeTracker *self)
+{
+  g_return_val_if_fail (ADW_IS_SWIPE_TRACKER (self), FALSE);
+
+  return self->lower_overshoot;
+}
+
+/**
+ * adw_swipe_tracker_set_lower_overshoot:
+ * @self: a swipe tracker
+ * @overshoot: whether to allow swiping past the first available snap point
+ *
+ * Sets whether to allow swiping past the first available snap point.
+ *
+ * Since: 1.4
+ */
+void
+adw_swipe_tracker_set_lower_overshoot (AdwSwipeTracker *self,
+                                       gboolean         overshoot)
+{
+  g_return_if_fail (ADW_IS_SWIPE_TRACKER (self));
+
+  overshoot = !!overshoot;
+
+  if (self->lower_overshoot == overshoot)
+    return;
+
+  self->lower_overshoot = overshoot;
+
+  g_object_notify_by_pspec (G_OBJECT (self), props[PROP_LOWER_OVERSHOOT]);
+}
+
+/**
+ * adw_swipe_tracker_get_upper_overshoot:
+ * @self: a swipe tracker
+ *
+ * Gets whether to allow swiping past the last available snap point.
+ *
+ * Returns: whether to allow swiping past the last available snap point
+ *
+ * Since: 1.4
+ */
+gboolean
+adw_swipe_tracker_get_upper_overshoot (AdwSwipeTracker *self)
+{
+  g_return_val_if_fail (ADW_IS_SWIPE_TRACKER (self), FALSE);
+
+  return self->upper_overshoot;
+}
+
+/**
+ * adw_swipe_tracker_set_upper_overshoot:
+ * @self: a swipe tracker
+ * @overshoot: whether to allow swiping past the last available snap point
+ *
+ * Sets whether to allow swiping past the last available snap point.
+ *
+ * Since: 1.4
+ */
+void
+adw_swipe_tracker_set_upper_overshoot (AdwSwipeTracker *self,
+                                       gboolean         overshoot)
+{
+  g_return_if_fail (ADW_IS_SWIPE_TRACKER (self));
+
+  overshoot = !!overshoot;
+
+  if (self->upper_overshoot == overshoot)
+    return;
+
+  self->upper_overshoot = overshoot;
+
+  g_object_notify_by_pspec (G_OBJECT (self), props[PROP_UPPER_OVERSHOOT]);
+}
+
+/**
+ * adw_swipe_tracker_get_allow_window_handle:
+ * @self: a swipe tracker
+ *
+ * Gets whether to allow touchscreen swiping from `GtkWindowHandle`.
+ *
+ * Returns: whether swiping from window handles is allowed
+ *
+ * Since: 1.5
+ */
+gboolean
+adw_swipe_tracker_get_allow_window_handle (AdwSwipeTracker *self)
+{
+  g_return_val_if_fail (ADW_IS_SWIPE_TRACKER (self), FALSE);
+
+  return self->allow_window_handle;
+}
+
+/**
+ * adw_swipe_tracker_set_allow_window_handle:
+ * @self: a swipe tracker
+ * @allow_window_handle: whether to allow swiping from window handles
+ *
+ * Sets whether to allow touchscreen swiping from `GtkWindowHandle`.
+ *
+ * Setting it to `TRUE` will make dragging the window impossible.
+ *
+ * Since: 1.5
+ */
+void
+adw_swipe_tracker_set_allow_window_handle (AdwSwipeTracker *self,
+                                           gboolean         allow_window_handle)
+{
+  g_return_if_fail (ADW_IS_SWIPE_TRACKER (self));
+
+  allow_window_handle = !!allow_window_handle;
+
+  if (self->allow_window_handle == allow_window_handle)
+    return;
+
+  self->allow_window_handle = allow_window_handle;
+
+  g_object_notify_by_pspec (G_OBJECT (self), props[PROP_ALLOW_WINDOW_HANDLE]);
+}
+
 /**
  * adw_swipe_tracker_shift_position:
  * @self: a swipe tracker
@@ -1431,3 +1763,12 @@ adw_swipe_tracker_reset (AdwSwipeTracker *self)
   if (self->scroll_controller)
     gtk_event_controller_reset (self->scroll_controller);
 }
+
+void
+adw_swipe_tracker_set_ignore_direction (AdwSwipeTracker *self,
+                                        gboolean         ignore_direction)
+{
+  g_return_if_fail (ADW_IS_SWIPE_TRACKER (self));
+
+  self->ignore_direction = TRUE;
+}
