@@ -3,7 +3,7 @@
  *
  * SPDX-License-Identifier: LGPL-2.1-or-later
  *
- * Author: Alexander Mikhaylenko <alexander.mikhaylenko@puri.sm>
+ * Author: Alice Mikhaylenko <alice.mikhaylenko@puri.sm>
  */
 
 #include "config.h"
@@ -11,6 +11,7 @@
 
 #include "adw-fading-label-private.h"
 #include "adw-gizmo-private.h"
+#include "adw-spinner-paintable.h"
 #include "adw-tab-view-private.h"
 #include "adw-timed-animation.h"
 
@@ -25,9 +26,7 @@ struct _AdwTabThumbnail
   GtkWidget *icon_title_box;
   GtkWidget *overlay;
   GtkPicture *picture;
-  GtkWidget *icon_stack;
-  GtkImage *icon;
-  GtkSpinner *spinner;
+  GtkWidget *icon;
   GtkImage *indicator_icon;
   GtkWidget *indicator_btn;
   GtkWidget *close_btn;
@@ -40,6 +39,7 @@ struct _AdwTabThumbnail
   AdwTabView *view;
   AdwTabPage *page;
   gboolean pinned;
+  gboolean loading;
 
   gboolean inverted;
 
@@ -91,34 +91,30 @@ update_tooltip (AdwTabThumbnail *self)
                                  adw_tab_page_get_title (page));
 }
 
-static void
-update_spinner (AdwTabThumbnail *self)
-{
-  gboolean loading = self->page && adw_tab_page_get_loading (self->page);
-  gboolean mapped = gtk_widget_get_mapped (GTK_WIDGET (self));
-
-  /* Don't use CPU when not needed */
-  gtk_spinner_set_spinning (self->spinner, loading && mapped);
-}
-
 static void
 update_icon (AdwTabThumbnail *self)
 {
   GIcon *gicon = adw_tab_page_get_icon (self->page);
   gboolean loading = adw_tab_page_get_loading (self->page);
-  const char *name = loading ? "spinner" : "icon";
 
-  gtk_image_set_from_gicon (self->icon, gicon);
-  gtk_widget_set_visible (self->icon_stack,
-                          (gicon != NULL || loading));
-  gtk_stack_set_visible_child_name (GTK_STACK (self->icon_stack), name);
+  if (loading && !self->loading) {
+    AdwSpinnerPaintable *paintable = adw_spinner_paintable_new (self->icon);
+
+    gtk_image_set_from_paintable (GTK_IMAGE (self->icon), GDK_PAINTABLE (paintable));
+
+    g_object_unref (paintable);
+  } else if (!loading) {
+    gtk_image_set_from_gicon (GTK_IMAGE (self->icon), gicon);
+  }
+
+  self->loading = loading;
+  gtk_widget_set_visible (self->icon, (gicon != NULL || loading));
 }
 
 static void
 update_loading (AdwTabThumbnail *self)
 {
   update_icon (self);
-  update_spinner (self);
   set_style_class (GTK_WIDGET (self), "loading",
                    adw_tab_page_get_loading (self->page));
 }
@@ -144,12 +140,11 @@ update_indicator (AdwTabThumbnail *self)
   set_style_class (GTK_WIDGET (self), "indicator", indicator != NULL);
 }
 
-static gboolean
+static void
 close_idle_cb (AdwTabThumbnail *self)
 {
   adw_tab_view_close_page (self->view, self->page);
-
-  return G_SOURCE_REMOVE;
+  g_object_unref (self);
 }
 
 static void
@@ -161,15 +156,14 @@ close_clicked_cb (AdwTabThumbnail *self)
   /* When animations are disabled, we don't want to immediately remove the
    * whole tab mid-click. Instead, defer it until the click has happened.
    */
-  g_idle_add ((GSourceFunc) close_idle_cb, self);
+  g_idle_add_once ((GSourceOnceFunc) close_idle_cb, g_object_ref (self));
 }
 
-static gboolean
+static void
 unpin_idle_cb (AdwTabThumbnail *self)
 {
   adw_tab_view_set_page_pinned (self->view, self->page, FALSE);
-
-  return G_SOURCE_REMOVE;
+  g_object_unref (self);
 }
 
 static void
@@ -181,7 +175,7 @@ unpin_clicked_cb (AdwTabThumbnail *self)
   /* When animations are disabled, we don't want to immediately unpin the
    * whole tab mid-click. Instead, defer it until the click has happened.
    */
-  g_idle_add ((GSourceFunc) unpin_idle_cb, self);
+  g_idle_add_once ((GSourceOnceFunc) unpin_idle_cb, g_object_ref (self));
 }
 
 static void
@@ -213,8 +207,10 @@ drop_cb (AdwTabThumbnail *self,
          GValue          *value)
 {
   gboolean ret = GDK_EVENT_PROPAGATE;
+  GdkDrop *drop = gtk_drop_target_get_current_drop (self->drop_target);
+  GdkDragAction preferred_action = gdk_drop_get_actions (drop);
 
-  g_signal_emit (self, signals[SIGNAL_EXTRA_DRAG_DROP], 0, value, &ret);
+  g_signal_emit (self, signals[SIGNAL_EXTRA_DRAG_DROP], 0, value, preferred_action, &ret);
 
   return ret;
 }
@@ -260,26 +256,6 @@ fade_animation_value_cb (double           value,
   gtk_widget_set_opacity (self->needs_attention_revealer, value);
 }
 
-static void
-adw_tab_thumbnail_map (GtkWidget *widget)
-{
-  AdwTabThumbnail *self = ADW_TAB_THUMBNAIL (widget);
-
-  GTK_WIDGET_CLASS (adw_tab_thumbnail_parent_class)->map (widget);
-
-  update_spinner (self);
-}
-
-static void
-adw_tab_thumbnail_unmap (GtkWidget *widget)
-{
-  AdwTabThumbnail *self = ADW_TAB_THUMBNAIL (widget);
-
-  GTK_WIDGET_CLASS (adw_tab_thumbnail_parent_class)->unmap (widget);
-
-  update_spinner (self);
-}
-
 static void
 measure_pinned_tab (AdwGizmo       *gizmo,
                     GtkOrientation  orientation,
@@ -490,9 +466,6 @@ adw_tab_thumbnail_class_init (AdwTabThumbnailClass *klass)
   object_class->get_property = adw_tab_thumbnail_get_property;
   object_class->set_property = adw_tab_thumbnail_set_property;
 
-  widget_class->map = adw_tab_thumbnail_map;
-  widget_class->unmap = adw_tab_thumbnail_unmap;
-
   props[PROP_VIEW] =
     g_param_spec_object ("view", NULL, NULL,
                          ADW_TYPE_TAB_VIEW,
@@ -523,8 +496,9 @@ adw_tab_thumbnail_class_init (AdwTabThumbnailClass *klass)
                   g_signal_accumulator_first_wins,
                   NULL, NULL,
                   G_TYPE_BOOLEAN,
-                  1,
-                  G_TYPE_VALUE);
+                  2,
+                  G_TYPE_VALUE,
+                  GDK_TYPE_DRAG_ACTION);
 
   signals[SIGNAL_EXTRA_DRAG_VALUE] =
     g_signal_new ("extra-drag-value",
@@ -543,9 +517,7 @@ adw_tab_thumbnail_class_init (AdwTabThumbnailClass *klass)
   gtk_widget_class_bind_template_child (widget_class, AdwTabThumbnail, overlay);
   gtk_widget_class_bind_template_child (widget_class, AdwTabThumbnail, icon_title_box);
   gtk_widget_class_bind_template_child (widget_class, AdwTabThumbnail, picture);
-  gtk_widget_class_bind_template_child (widget_class, AdwTabThumbnail, icon_stack);
   gtk_widget_class_bind_template_child (widget_class, AdwTabThumbnail, icon);
-  gtk_widget_class_bind_template_child (widget_class, AdwTabThumbnail, spinner);
   gtk_widget_class_bind_template_child (widget_class, AdwTabThumbnail, indicator_icon);
   gtk_widget_class_bind_template_child (widget_class, AdwTabThumbnail, indicator_btn);
   gtk_widget_class_bind_template_child (widget_class, AdwTabThumbnail, close_btn);
@@ -580,6 +552,8 @@ adw_tab_thumbnail_init (AdwTabThumbnail *self)
                                                   0, 1,
                                                   FADE_TRANSITION_DURATION,
                                                   target);
+
+  adw_timed_animation_set_easing (ADW_TIMED_ANIMATION (self->fade_animation), ADW_EASE);
 }
 
 AdwTabThumbnail *
@@ -627,7 +601,6 @@ adw_tab_thumbnail_set_page (AdwTabThumbnail *self,
     gtk_picture_set_paintable (GTK_PICTURE (self->picture), paintable);
 
     update_tooltip (self);
-    update_spinner (self);
     update_icon (self);
     update_indicator (self);
     update_loading (self);
@@ -732,6 +705,8 @@ adw_tab_thumbnail_fade_out (AdwTabThumbnail *self)
   g_return_if_fail (ADW_IS_TAB_THUMBNAIL (self));
 
   adw_animation_reset (self->fade_animation);
+
+  gtk_widget_set_opacity (self->overlay, 0);
 }
 
 void
@@ -739,5 +714,7 @@ adw_tab_thumbnail_fade_in (AdwTabThumbnail *self)
 {
   g_return_if_fail (ADW_IS_TAB_THUMBNAIL (self));
 
+  gtk_widget_set_opacity (self->overlay, 1);
+
   adw_animation_play (self->fade_animation);
 }
