@@ -36,6 +36,9 @@
  * main child, on top of which it can display a [class@Toast], overlaid.
  * Toasts can be shown with [method@ToastOverlay.add_toast].
  *
+ * Use [method@ToastOverlay.dismiss_all] to dismiss all toasts at once, or
+ * [method@Toast.dismiss] to dismiss a single toast.
+ *
  * See [class@Toast] for details.
  *
  * ## CSS nodes
@@ -63,7 +66,7 @@
  *
  * ## Accessibility
  *
- * `AdwToastOverlay` uses the `GTK_ACCESSIBLE_ROLE_TAB_GROUP` role.
+ * `AdwToastOverlay` uses the [enum@Gtk.AccessibleRole.GROUP] role.
  */
 
 typedef struct {
@@ -191,6 +194,8 @@ hide_current_toast (AdwToastOverlay *self)
   info->hide_animation =
     adw_timed_animation_new (GTK_WIDGET (self), 1, 0, HIDE_DURATION, target);
 
+  adw_timed_animation_set_easing (ADW_TIMED_ANIMATION (info->hide_animation), ADW_EASE);
+
   g_signal_connect_swapped (info->hide_animation, "done",
                             G_CALLBACK (hide_done_cb), info);
 
@@ -244,7 +249,14 @@ show_toast (AdwToastOverlay *self,
   const char *title, *button_label;
   char *announcement;
 
-  g_assert (!info->widget);
+  if (info->widget) {
+    /* The toast's hide animation is currently playing. Complete it immediately.
+     * hide_done_cb() will clear the widget.
+     */
+    g_assert (info->hide_animation);
+    adw_animation_skip (info->hide_animation);
+    g_assert (!info->widget);
+  }
 
   self->current_toast = info;
 
@@ -259,6 +271,8 @@ show_toast (AdwToastOverlay *self,
                              self->hiding_toasts ? REPLACE_DURATION : SHOW_DURATION,
                              target);
 
+  adw_timed_animation_set_easing (ADW_TIMED_ANIMATION (info->show_animation), ADW_EASE);
+
   info->shown_id = g_signal_connect_swapped (info->show_animation, "done",
                                              G_CALLBACK (show_done_cb), info);
 
@@ -614,12 +628,12 @@ adw_toast_overlay_set_child (AdwToastOverlay *self,
   g_return_if_fail (ADW_IS_TOAST_OVERLAY (self));
   g_return_if_fail (child == NULL || GTK_IS_WIDGET (child));
 
-  if (child)
-    g_return_if_fail (gtk_widget_get_parent (child) == NULL);
-
   if (self->child == child)
     return;
 
+  if (child)
+    g_return_if_fail (gtk_widget_get_parent (child) == NULL);
+
   if (self->child)
     gtk_widget_unparent (self->child);
 
@@ -712,3 +726,22 @@ adw_toast_overlay_add_toast (AdwToastOverlay *self,
     g_assert_not_reached ();
   }
 }
+
+/**
+ * adw_toast_overlay_dismiss_all:
+ * @self: a toast overlay
+ *
+ * Dismisses all displayed toasts.
+ *
+ * Use [method@Toast.dismiss] to dismiss a single toast.
+ *
+ * Since: 1.7
+ */
+void
+adw_toast_overlay_dismiss_all (AdwToastOverlay *self)
+{
+  g_return_if_fail (ADW_IS_TOAST_OVERLAY (self));
+
+  g_clear_pointer (&self->current_toast, dismiss_and_free_toast_info);
+  g_queue_clear_full (self->queue, (GDestroyNotify) dismiss_and_free_toast_info);
+}
