@@ -16,6 +16,7 @@
 
 #include "adw-animation-util.h"
 #include "adw-gizmo-private.h"
+#include "adw-timed-animation.h"
 #include "adw-widget-utils-private.h"
 
 /**
@@ -37,7 +38,9 @@
  * [property@ViewStackPage:needs-attention], and
  * [property@ViewStackPage:badge-number] properties.
  *
- * Unlike [class@Gtk.Stack], transitions between views are not animated.
+ * Unlike [class@Gtk.Stack], transitions between views can only be animated via
+ * a crossfade and size changes are always interpolated. Animations are disabled
+ * by default. Use [property@ViewStack:enable-transitions] to enable them.
  *
  * `AdwViewStack` maintains a [class@ViewStackPage] object for each added child,
  * which holds additional per-child properties. You obtain the
@@ -101,6 +104,9 @@ enum {
   PROP_VHOMOGENEOUS,
   PROP_VISIBLE_CHILD,
   PROP_VISIBLE_CHILD_NAME,
+  PROP_ENABLE_TRANSITIONS,
+  PROP_TRANSITION_DURATION,
+  PROP_TRANSITION_RUNNING,
   PROP_PAGES,
   LAST_PROP
 };
@@ -168,6 +174,15 @@ struct _AdwViewStack {
 
   gboolean homogeneous[2];
 
+  gboolean enable_transitions;
+  guint transition_duration;
+  AdwViewStackPage *last_visible_child;
+  gboolean transition_running;
+  AdwAnimation *animation;
+
+  int last_visible_widget_width;
+  int last_visible_widget_height;
+
   GtkSelectionModel *pages;
 };
 
@@ -235,6 +250,9 @@ adw_view_stack_page_set_property (GObject      *object,
   switch (property_id) {
   case PAGE_PROP_CHILD:
     g_set_object (&self->widget, g_value_get_object (value));
+    if (self->widget)
+      gtk_accessible_set_accessible_parent (GTK_ACCESSIBLE (self->widget),
+                                            GTK_ACCESSIBLE (self), NULL);
     break;
   case PAGE_PROP_NAME:
     adw_view_stack_page_set_name (self, g_value_get_string (value));
@@ -286,10 +304,7 @@ adw_view_stack_page_finalize (GObject *object)
   g_clear_pointer (&self->name, g_free);
   g_clear_pointer (&self->title, g_free);
   g_clear_pointer (&self->icon_name, g_free);
-
-  if (self->last_focus)
-    g_object_remove_weak_pointer (G_OBJECT (self->last_focus),
-                                  (gpointer *) &self->last_focus);
+  g_clear_weak_pointer (&self->last_focus);
 
   G_OBJECT_CLASS (adw_view_stack_page_parent_class)->finalize (object);
 }
@@ -305,7 +320,7 @@ adw_view_stack_page_class_init (AdwViewStackPageClass *class)
   object_class->set_property = adw_view_stack_page_set_property;
 
   /**
-   * AdwViewStackPage:child: (attributes org.gtk.Property.get=adw_view_stack_page_get_child)
+   * AdwViewStackPage:child:
    *
    * The stack child to which the page belongs.
    */
@@ -315,7 +330,7 @@ adw_view_stack_page_class_init (AdwViewStackPageClass *class)
                          G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY | G_PARAM_STATIC_STRINGS);
 
   /**
-   * AdwViewStackPage:name: (attributes org.gtk.Property.get=adw_view_stack_page_get_name org.gtk.Property.set=adw_view_stack_page_set_name)
+   * AdwViewStackPage:name:
    *
    * The name of the child page.
    */
@@ -325,7 +340,7 @@ adw_view_stack_page_class_init (AdwViewStackPageClass *class)
                          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * AdwViewStackPage:title: (attributes org.gtk.Property.get=adw_view_stack_page_get_title org.gtk.Property.set=adw_view_stack_page_set_title)
+   * AdwViewStackPage:title:
    *
    * The title of the child page.
    */
@@ -335,7 +350,7 @@ adw_view_stack_page_class_init (AdwViewStackPageClass *class)
                          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * AdwViewStackPage:use-underline: (attributes org.gtk.Property.get=adw_view_stack_page_get_use_underline org.gtk.Property.set=adw_view_stack_page_set_use_underline)
+   * AdwViewStackPage:use-underline:
    *
    * Whether an embedded underline in the title indicates a mnemonic.
    */
@@ -345,7 +360,7 @@ adw_view_stack_page_class_init (AdwViewStackPageClass *class)
                           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * AdwViewStackPage:icon-name: (attributes org.gtk.Property.get=adw_view_stack_page_get_icon_name org.gtk.Property.set=adw_view_stack_page_set_icon_name)
+   * AdwViewStackPage:icon-name:
    *
    * The icon name of the child page.
    */
@@ -355,7 +370,7 @@ adw_view_stack_page_class_init (AdwViewStackPageClass *class)
                          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * AdwViewStackPage:needs-attention: (attributes org.gtk.Property.get=adw_view_stack_page_get_needs_attention org.gtk.Property.set=adw_view_stack_page_set_needs_attention)
+   * AdwViewStackPage:needs-attention:
    *
    * Whether the page requires the user attention.
    *
@@ -367,7 +382,7 @@ adw_view_stack_page_class_init (AdwViewStackPageClass *class)
                           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * AdwViewStackPage:badge-number: (attributes org.gtk.Property.get=adw_view_stack_page_get_badge_number org.gtk.Property.set=adw_view_stack_page_set_badge_number)
+   * AdwViewStackPage:badge-number:
    *
    * The badge number for this page.
    *
@@ -382,7 +397,7 @@ adw_view_stack_page_class_init (AdwViewStackPageClass *class)
                        G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * AdwViewStackPage:visible: (attributes org.gtk.Property.get=adw_view_stack_page_get_visible org.gtk.Property.set=adw_view_stack_page_set_visible)
+   * AdwViewStackPage:visible:
    *
    * Whether this page is visible.
    *
@@ -576,7 +591,7 @@ G_DEFINE_FINAL_TYPE_WITH_CODE (AdwViewStackPages, adw_view_stack_pages, G_TYPE_O
                                G_IMPLEMENT_INTERFACE (GTK_TYPE_SELECTION_MODEL, adw_view_stack_pages_selection_model_init))
 
 /**
- * adw_view_stack_pages_get_selected_page: (attributes org.gtk.Method.get_property=selected-page)
+ * adw_view_stack_pages_get_selected_page:
  * @self: a [class@ViewStackPages]
  *
  * Gets the [class@ViewStackPage] for the visible child of a view stack
@@ -606,7 +621,7 @@ adw_view_stack_pages_get_selected_page (AdwViewStackPages *self)
 }
 
 /**
- * adw_view_stack_pages_set_selected_page: (attributes org.gtk.Method.set_property=selected-page)
+ * adw_view_stack_pages_set_selected_page:
  * @self: a [class@ViewStackPages]
  * @page: a stack page within the associated stack
  *
@@ -685,7 +700,7 @@ adw_view_stack_pages_class_init (AdwViewStackPagesClass *class)
   object_class->set_property = adw_view_stack_pages_set_property;
 
   /**
-   * AdwViewStackPages:selected-page: (attributes org.gtk.Property.get=adw_view_stack_pages_get_selected_page org.gtk.Property.set=adw_view_stack_pages_set_selected_page)
+   * AdwViewStackPages:selected-page:
    *
    * The selected [class@ViewStackPage] within the [class@ViewStackPages].
    *
@@ -761,6 +776,42 @@ find_page_for_name (AdwViewStack *self,
   return NULL;
 }
 
+static void
+set_transition_running (AdwViewStack *self,
+                        gboolean      running)
+{
+  if (self->transition_running == running)
+    return;
+
+  self->transition_running = running;
+  g_object_notify_by_pspec (G_OBJECT (self), props[PROP_TRANSITION_RUNNING]);
+}
+
+static void
+transition_cb (double        value,
+               AdwViewStack *self)
+{
+  if (self->homogeneous[GTK_ORIENTATION_HORIZONTAL] &&
+      self->homogeneous[GTK_ORIENTATION_VERTICAL]) {
+    gtk_widget_queue_draw (GTK_WIDGET (self));
+  } else {
+    gtk_widget_queue_resize (GTK_WIDGET (self));
+  }
+}
+
+static void
+transition_done_cb (AdwViewStack *self)
+{
+  if (self->last_visible_child) {
+    gtk_widget_set_child_visible (self->last_visible_child->widget, FALSE);
+    self->last_visible_child = NULL;
+  }
+
+  adw_animation_reset (self->animation);
+
+  set_transition_running (self, FALSE);
+}
+
 static void
 set_visible_child (AdwViewStack     *self,
                    AdwViewStackPage *page)
@@ -821,16 +872,21 @@ set_visible_child (AdwViewStack     *self,
       gtk_widget_is_ancestor (focus, self->visible_child->widget)) {
     contains_focus = TRUE;
 
-    if (self->visible_child->last_focus)
-      g_object_remove_weak_pointer (G_OBJECT (self->visible_child->last_focus),
-                                    (gpointer *)&self->visible_child->last_focus);
-    self->visible_child->last_focus = focus;
-    g_object_add_weak_pointer (G_OBJECT (self->visible_child->last_focus),
-                               (gpointer *)&self->visible_child->last_focus);
+    g_set_weak_pointer (&self->visible_child->last_focus, focus);
   }
 
-  if (self->visible_child && self->visible_child->widget)
-    gtk_widget_set_child_visible (self->visible_child->widget, FALSE);
+  if (self->transition_running)
+    adw_animation_skip (self->animation);
+
+  if (self->visible_child && self->visible_child->widget) {
+    if (gtk_widget_is_visible (widget)) {
+      self->last_visible_child = self->visible_child;
+      self->last_visible_widget_width = gtk_widget_get_width (widget);
+      self->last_visible_widget_height = gtk_widget_get_height (widget);
+    } else {
+      gtk_widget_set_child_visible (self->visible_child->widget, FALSE);
+    }
+  }
 
   self->visible_child = page;
 
@@ -868,6 +924,9 @@ set_visible_child (AdwViewStack     *self,
                                              MIN (old_pos, new_pos),
                                              MAX (old_pos, new_pos) - MIN (old_pos, new_pos) + 1);
   }
+
+  set_transition_running (self, TRUE);
+  adw_animation_play (self->animation);
 }
 
 static void
@@ -883,6 +942,11 @@ update_child_visible (AdwViewStack     *self,
   else if (self->visible_child == page && !visible)
     set_visible_child (self, NULL);
 
+  if (page == self->last_visible_child) {
+    gtk_widget_set_child_visible (self->last_visible_child->widget, FALSE);
+    self->last_visible_child = NULL;
+  }
+
   gtk_accessible_update_state (GTK_ACCESSIBLE (page),
                                GTK_ACCESSIBLE_STATE_HIDDEN, !visible,
                                -1);
@@ -1020,6 +1084,33 @@ stack_remove (AdwViewStack  *self,
     gtk_widget_queue_resize (GTK_WIDGET (self));
 }
 
+static void
+adw_view_stack_snapshot (GtkWidget   *widget,
+                         GtkSnapshot *snapshot)
+{
+  AdwViewStack *self = ADW_VIEW_STACK (widget);
+
+  if (!self->visible_child)
+    return;
+
+  if (adw_animation_get_state (self->animation) == ADW_ANIMATION_PLAYING) {
+    double t = adw_animation_get_value (self->animation);
+
+    gtk_snapshot_push_cross_fade (snapshot, t);
+
+    if (self->last_visible_child)
+      gtk_widget_snapshot_child (widget, self->last_visible_child->widget, snapshot);
+    gtk_snapshot_pop (snapshot);
+
+    gtk_widget_snapshot_child (widget, self->visible_child->widget, snapshot);
+    gtk_snapshot_pop (snapshot);
+
+    return;
+  }
+
+  gtk_widget_snapshot_child (widget, self->visible_child->widget, snapshot);
+}
+
 static void
 adw_view_stack_size_allocate (GtkWidget *widget,
                               int        width,
@@ -1028,10 +1119,47 @@ adw_view_stack_size_allocate (GtkWidget *widget,
 {
   AdwViewStack *self = ADW_VIEW_STACK (widget);
 
-  if (!self->visible_child)
-    return;
+  if (self->last_visible_child) {
+    GtkAllocation child_allocation;
 
-  gtk_widget_allocate (self->visible_child->widget, width, height, baseline, NULL);
+    child_allocation.x = 0;
+    child_allocation.y = 0;
+    child_allocation.width = width;
+    child_allocation.height = height;
+
+    adw_ensure_child_allocation_size (self->last_visible_child->widget, &child_allocation);
+
+    gtk_widget_size_allocate (self->last_visible_child->widget, &child_allocation, -1);
+  }
+
+  if (self->visible_child) {
+    GtkAllocation child_allocation;
+
+    child_allocation.x = 0;
+    child_allocation.y = 0;
+    child_allocation.width = width;
+    child_allocation.height = height;
+
+    adw_ensure_child_allocation_size (self->visible_child->widget, &child_allocation);
+
+    gtk_widget_size_allocate (self->visible_child->widget, &child_allocation, -1);
+  }
+}
+
+/* Given adw_lerp (a, b, t) -> r, find b. In other words: we know what size
+ * we're interpolating *from*, the current size, and the current interpolation
+ * progress; guess which size we're interpolating *to*.
+ *
+ * Note that this is different from the similarly named inverse_lerp function
+ * in adw-clamp-layout.c. That one finds t given a, b, and r, while this one
+ * finds b given a, r, and t.
+ */
+static inline double
+inverse_lerp (int a,
+              int r,
+              double t)
+{
+  return (r - a * (1.0 - t)) / t;
 }
 
 static void
@@ -1044,9 +1172,47 @@ adw_view_stack_measure (GtkWidget      *widget,
                         int            *natural_baseline)
 {
   AdwViewStack *self = ADW_VIEW_STACK (widget);
-  int child_min, child_nat;
+  int child_min, child_nat, child_for_size;
+  int last_size, last_opposite_size;
+  double t;
   GList *l;
 
+  if (self->last_visible_child != NULL && !self->homogeneous[orientation]) {
+    t = adw_animation_get_value (self->animation);
+
+    if (orientation == GTK_ORIENTATION_HORIZONTAL) {
+      last_size = self->last_visible_widget_width;
+      last_opposite_size = self->last_visible_widget_height;
+    } else {
+      last_size = self->last_visible_widget_height;
+      last_opposite_size = self->last_visible_widget_width;
+    }
+
+    /* Work out which for_size we want to pass for measuring our children */
+    if (for_size == -1 || for_size == last_opposite_size ||
+        self->homogeneous[OPPOSITE_ORIENTATION(orientation)])
+      child_for_size = for_size;
+    else if (t <= 0.0)  /* avoid -Wfloat-equal */
+      /* We're going to return last_size anyway */
+      child_for_size = -1;
+    else {
+      double d = inverse_lerp (last_opposite_size, for_size, t);
+      /* inverse_lerp is numerically unstable due to its use of floating-point
+       * division, potentially by a very small value when progress is close to
+       * zero. So we sanity check the return value.
+       */
+      if (isnan (d) || isinf (d) || d < 0 || d >= G_MAXINT)
+        child_for_size = -1;
+      else
+        child_for_size = floor (d);
+    }
+  } else {
+    child_for_size = for_size;
+    /* Avoid -Wmaybe-uninitialized */
+    last_size = 0;
+    t = 1.0;
+  }
+
   *minimum = 0;
   *natural = 0;
 
@@ -1058,19 +1224,39 @@ adw_view_stack_measure (GtkWidget      *widget,
         self->visible_child != page)
       continue;
 
-    if (gtk_widget_get_visible (child)) {
-      if (!self->homogeneous[OPPOSITE_ORIENTATION(orientation)] && self->visible_child != page) {
-        int min_for_size;
+    if (!gtk_widget_get_visible (child))
+      continue;
 
-        gtk_widget_measure (child, OPPOSITE_ORIENTATION (orientation), -1, &min_for_size, NULL, NULL, NULL);
-        gtk_widget_measure (child, orientation, MAX (min_for_size, for_size), &child_min, &child_nat, NULL, NULL);
-      } else {
-        gtk_widget_measure (child, orientation, for_size, &child_min, &child_nat, NULL, NULL);
+    if (!self->homogeneous[OPPOSITE_ORIENTATION(orientation)] && self->visible_child != page) {
+      int measure_for_size;
+
+      if (child_for_size == -1)
+        measure_for_size = -1;
+      else {
+        gtk_widget_measure (child, OPPOSITE_ORIENTATION (orientation),
+                            -1,
+                            &measure_for_size, NULL,
+                            NULL, NULL);
+        measure_for_size = MAX (measure_for_size, child_for_size);
       }
-
-      *minimum = MAX (*minimum, child_min);
-      *natural = MAX (*natural, child_nat);
+      gtk_widget_measure (child, orientation,
+                          measure_for_size,
+                          &child_min, &child_nat,
+                          NULL, NULL);
+    } else {
+      gtk_widget_measure (child, orientation,
+                          child_for_size,
+                          &child_min, &child_nat,
+                          NULL, NULL);
     }
+
+    *minimum = MAX (*minimum, child_min);
+    *natural = MAX (*natural, child_nat);
+  }
+
+  if (self->last_visible_child != NULL && !self->homogeneous[orientation]) {
+    *minimum = ceil (adw_lerp (last_size, *minimum, t));
+    *natural = ceil (adw_lerp (last_size, *natural, t));
   }
 }
 
@@ -1095,6 +1281,15 @@ adw_view_stack_get_property (GObject    *object,
   case PROP_VISIBLE_CHILD_NAME:
     g_value_set_string (value, adw_view_stack_get_visible_child_name (self));
     break;
+  case PROP_ENABLE_TRANSITIONS:
+    g_value_set_boolean (value, adw_view_stack_get_enable_transitions (self));
+    break;
+  case PROP_TRANSITION_DURATION:
+    g_value_set_uint (value, adw_view_stack_get_transition_duration (self));
+    break;
+  case PROP_TRANSITION_RUNNING:
+    g_value_set_boolean (value, adw_view_stack_get_transition_running (self));
+    break;
   case PROP_PAGES:
     g_value_take_object (value, adw_view_stack_get_pages (self));
     break;
@@ -1125,6 +1320,12 @@ adw_view_stack_set_property (GObject      *object,
   case PROP_VISIBLE_CHILD_NAME:
     adw_view_stack_set_visible_child_name (self, g_value_get_string (value));
     break;
+  case PROP_ENABLE_TRANSITIONS:
+    adw_view_stack_set_enable_transitions (self, g_value_get_boolean (value));
+    break;
+  case PROP_TRANSITION_DURATION:
+    adw_view_stack_set_transition_duration (self, g_value_get_uint (value));
+    break;
   default:
     G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
     break;
@@ -1152,9 +1353,7 @@ adw_view_stack_finalize (GObject *object)
 {
   AdwViewStack *self = ADW_VIEW_STACK (object);
 
-  if (self->pages)
-    g_object_remove_weak_pointer (G_OBJECT (self->pages),
-                                  (gpointer *) &self->pages);
+  g_clear_weak_pointer (&self->pages);
 
   G_OBJECT_CLASS (adw_view_stack_parent_class)->finalize (object);
 }
@@ -1170,13 +1369,14 @@ adw_view_stack_class_init (AdwViewStackClass *klass)
   object_class->dispose = adw_view_stack_dispose;
   object_class->finalize = adw_view_stack_finalize;
 
+  widget_class->snapshot = adw_view_stack_snapshot;
   widget_class->size_allocate = adw_view_stack_size_allocate;
   widget_class->measure = adw_view_stack_measure;
   widget_class->get_request_mode = adw_widget_get_request_mode;
   widget_class->compute_expand = adw_widget_compute_expand;
 
   /**
-   * AdwViewStack:hhomogeneous: (attributes org.gtk.Property.get=adw_view_stack_get_hhomogeneous org.gtk.Property.set=adw_view_stack_set_hhomogeneous)
+   * AdwViewStack:hhomogeneous:
    *
    * Whether the stack is horizontally homogeneous.
    *
@@ -1192,7 +1392,7 @@ adw_view_stack_class_init (AdwViewStackClass *klass)
                           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * AdwViewStack:vhomogeneous: (attributes org.gtk.Property.get=adw_view_stack_get_vhomogeneous org.gtk.Property.set=adw_view_stack_set_vhomogeneous)
+   * AdwViewStack:vhomogeneous:
    *
    * Whether the stack is vertically homogeneous.
    *
@@ -1208,7 +1408,7 @@ adw_view_stack_class_init (AdwViewStackClass *klass)
                           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * AdwViewStack:visible-child: (attributes org.gtk.Property.get=adw_view_stack_get_visible_child org.gtk.Property.set=adw_view_stack_set_visible_child)
+   * AdwViewStack:visible-child:
    *
    * The widget currently visible in the stack.
    */
@@ -1218,7 +1418,7 @@ adw_view_stack_class_init (AdwViewStackClass *klass)
                          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * AdwViewStack:visible-child-name: (attributes org.gtk.Property.get=adw_view_stack_get_visible_child_name org.gtk.Property.set=adw_view_stack_set_visible_child_name)
+   * AdwViewStack:visible-child-name:
    *
    * The name of the widget currently visible in the stack.
    *
@@ -1230,7 +1430,53 @@ adw_view_stack_class_init (AdwViewStackClass *klass)
                          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * AdwViewStack:pages: (attributes org.gtk.Property.get=adw_view_stack_get_pages)
+   * AdwViewStack:enable-transitions:
+   *
+   * Whether the stack uses a crossfade transition between pages.
+   *
+   * Use [property@ViewStack:transition-duration] to control the duration, and
+   * [property@ViewStack:transition-running] to know when the transition is
+   * running.
+   *
+   * Since: 1.7
+   */
+  props[PROP_ENABLE_TRANSITIONS] =
+    g_param_spec_boolean ("enable-transitions", NULL, NULL,
+                          FALSE,
+                          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
+
+  /**
+   * AdwViewStack:transition-duration:
+   *
+   * The transition animation duration, in milliseconds.
+   *
+   * Only used when [property@ViewStack:enable-transitions] is set to `TRUE`.
+   *
+   * Since: 1.7
+   */
+  props[PROP_TRANSITION_DURATION] =
+    g_param_spec_uint ("transition-duration", NULL, NULL,
+                       0, G_MAXUINT, 200,
+                       G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
+
+  /**
+   * AdwViewStack:transition-running:
+   *
+   * Whether a transition is currently running.
+   *
+   * If a transition is impossible, the property value will be set to `TRUE` and
+   * then immediately to `FALSE`, so it's possible to rely on its notifications
+   * to know that a transition has happened.
+   *
+   * Since: 1.7
+   */
+  props[PROP_TRANSITION_RUNNING] =
+    g_param_spec_boolean ("transition-running", NULL, NULL,
+                          FALSE,
+                          G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
+
+  /**
+   * AdwViewStack:pages:
    *
    * A selection model with the stack's pages.
    *
@@ -1252,8 +1498,20 @@ adw_view_stack_class_init (AdwViewStackClass *klass)
 static void
 adw_view_stack_init (AdwViewStack *self)
 {
+  AdwAnimationTarget *target;
+
   self->homogeneous[GTK_ORIENTATION_VERTICAL] = TRUE;
   self->homogeneous[GTK_ORIENTATION_HORIZONTAL] = TRUE;
+  self->transition_duration = 200;
+
+  target = adw_callback_animation_target_new ((AdwAnimationTargetFunc) transition_cb,
+                                              self, NULL);
+  self->animation = adw_timed_animation_new (GTK_WIDGET (self), 0, 1, 0, target);
+
+  adw_timed_animation_set_easing (ADW_TIMED_ANIMATION (self->animation), ADW_EASE);
+
+  g_signal_connect_swapped (self->animation, "done",
+                            G_CALLBACK (transition_done_cb), self);
 }
 
 static void
@@ -1298,7 +1556,7 @@ adw_view_stack_accessible_init (GtkAccessibleInterface *iface)
 }
 
 /**
- * adw_view_stack_page_get_child: (attributes org.gtk.Method.get_property=child)
+ * adw_view_stack_page_get_child:
  * @self: a view stack page
  *
  * Gets the stack child to which @self belongs.
@@ -1314,7 +1572,7 @@ adw_view_stack_page_get_child (AdwViewStackPage *self)
 }
 
 /**
- * adw_view_stack_page_get_name: (attributes org.gtk.Method.get_property=name)
+ * adw_view_stack_page_get_name:
  * @self: a view stack page
  *
  * Gets the name of the page.
@@ -1330,7 +1588,7 @@ adw_view_stack_page_get_name (AdwViewStackPage *self)
 }
 
 /**
- * adw_view_stack_page_set_name: (attributes org.gtk.Method.set_property=name)
+ * adw_view_stack_page_set_name:
  * @self: a view stack page
  * @name: (nullable): the page name
  *
@@ -1375,7 +1633,7 @@ adw_view_stack_page_set_name (AdwViewStackPage *self,
 }
 
 /**
- * adw_view_stack_page_get_title: (attributes org.gtk.Method.get_property=title)
+ * adw_view_stack_page_get_title:
  * @self: a view stack page
  *
  * Gets the page title.
@@ -1391,7 +1649,7 @@ adw_view_stack_page_get_title (AdwViewStackPage *self)
 }
 
 /**
- * adw_view_stack_page_set_title: (attributes org.gtk.Method.set_property=title)
+ * adw_view_stack_page_set_title:
  * @self: a view stack page
  * @title: (nullable): the page title
  *
@@ -1414,7 +1672,7 @@ adw_view_stack_page_set_title (AdwViewStackPage *self,
 }
 
 /**
- * adw_view_stack_page_get_use_underline: (attributes org.gtk.Method.get_property=use-underline)
+ * adw_view_stack_page_get_use_underline:
  * @self: a view stack page
  *
  * Gets whether underlines in the page title indicate mnemonics.
@@ -1428,7 +1686,7 @@ adw_view_stack_page_get_use_underline (AdwViewStackPage *self)
 }
 
 /**
- * adw_view_stack_page_set_use_underline: (attributes org.gtk.Method.set_property=use-underline)
+ * adw_view_stack_page_set_use_underline:
  * @self: a view stack page
  * @use_underline: the new value to set
  *
@@ -1449,7 +1707,7 @@ adw_view_stack_page_set_use_underline (AdwViewStackPage *self,
 }
 
 /**
- * adw_view_stack_page_get_icon_name: (attributes org.gtk.Method.get_property=icon-name)
+ * adw_view_stack_page_get_icon_name:
  * @self: a view stack page
  *
  * Gets the icon name of the page.
@@ -1465,7 +1723,7 @@ adw_view_stack_page_get_icon_name (AdwViewStackPage *self)
 }
 
 /**
- * adw_view_stack_page_set_icon_name: (attributes org.gtk.Method.set_property=icon-name)
+ * adw_view_stack_page_set_icon_name:
  * @self: a view stack page
  * @icon_name: (nullable): the icon name
  *
@@ -1484,7 +1742,7 @@ adw_view_stack_page_set_icon_name (AdwViewStackPage *self,
 }
 
 /**
- * adw_view_stack_page_get_needs_attention: (attributes org.gtk.Method.get_property=needs-attention)
+ * adw_view_stack_page_get_needs_attention:
  * @self: a view stack page
  *
  * Gets whether the page requires the user attention.
@@ -1500,7 +1758,7 @@ adw_view_stack_page_get_needs_attention (AdwViewStackPage *self)
 }
 
 /**
- * adw_view_stack_page_set_needs_attention: (attributes org.gtk.Method.set_property=needs-attention)
+ * adw_view_stack_page_set_needs_attention:
  * @self: a view stack page
  * @needs_attention: the new value to set
  *
@@ -1525,7 +1783,7 @@ adw_view_stack_page_set_needs_attention (AdwViewStackPage *self,
 }
 
 /**
- * adw_view_stack_page_get_badge_number: (attributes org.gtk.Method.get_property=badge-number)
+ * adw_view_stack_page_get_badge_number:
  * @self: a view stack page
  *
  * Gets the badge number for this page.
@@ -1541,7 +1799,7 @@ adw_view_stack_page_get_badge_number (AdwViewStackPage *self)
 }
 
 /**
- * adw_view_stack_page_set_badge_number: (attributes org.gtk.Method.set_property=badge-number)
+ * adw_view_stack_page_set_badge_number:
  * @self: a view stack page
  * @badge_number: the new value to set
  *
@@ -1567,7 +1825,7 @@ adw_view_stack_page_set_badge_number (AdwViewStackPage *self,
 }
 
 /**
- * adw_view_stack_page_get_visible: (attributes org.gtk.Method.get_property=visible)
+ * adw_view_stack_page_get_visible:
  * @self: a view stack page
  *
  * Gets whether @self is visible in its `AdwViewStack`.
@@ -1586,7 +1844,7 @@ adw_view_stack_page_get_visible (AdwViewStackPage *self)
 }
 
 /**
- * adw_view_stack_page_set_visible: (attributes org.gtk.Method.set_property=visible)
+ * adw_view_stack_page_set_visible:
  * @self: a view stack page
  * @visible: whether @self is visible
  *
@@ -1803,10 +2061,10 @@ adw_view_stack_get_child_by_name (AdwViewStack *self,
 }
 
 /**
- * adw_view_stack_get_visible_child: (attributes org.gtk.Method.get_property=visible-child)
+ * adw_view_stack_get_visible_child:
  * @self: a view stack
  *
- * Gets the currently visible child of @self, .
+ * Gets the currently visible child of @self.
  *
  * Returns: (transfer none) (nullable): the visible child
  */
@@ -1819,7 +2077,7 @@ adw_view_stack_get_visible_child (AdwViewStack *self)
 }
 
 /**
- * adw_view_stack_set_visible_child: (attributes org.gtk.Method.set_property=visible-child)
+ * adw_view_stack_set_visible_child:
  * @self: a view stack
  * @child: a child of @self
  *
@@ -1847,7 +2105,7 @@ adw_view_stack_set_visible_child (AdwViewStack *self,
 }
 
 /**
- * adw_view_stack_get_visible_child_name: (attributes org.gtk.Method.get_property=visible-child-name)
+ * adw_view_stack_get_visible_child_name:
  * @self: a view stack
  *
  * Returns the name of the currently visible child of @self.
@@ -1863,7 +2121,7 @@ adw_view_stack_get_visible_child_name (AdwViewStack *self)
 }
 
 /**
- * adw_view_stack_set_visible_child_name: (attributes org.gtk.Method.set_property=visible-child-name)
+ * adw_view_stack_set_visible_child_name:
  * @self: a view stack
  * @name: the name of the child
  *
@@ -1895,7 +2153,7 @@ adw_view_stack_set_visible_child_name (AdwViewStack *self,
 }
 
 /**
- * adw_view_stack_get_hhomogeneous: (attributes org.gtk.Method.get_property=hhomogeneous)
+ * adw_view_stack_get_hhomogeneous:
  * @self: a view stack
  *
  * Gets whether @self is horizontally homogeneous.
@@ -1911,7 +2169,7 @@ adw_view_stack_get_hhomogeneous (AdwViewStack *self)
 }
 
 /**
- * adw_view_stack_set_hhomogeneous: (attributes org.gtk.Method.set_property=hhomogeneous)
+ * adw_view_stack_set_hhomogeneous:
  * @self: a view stack
  * @hhomogeneous: whether to make @self horizontally homogeneous
  *
@@ -1943,7 +2201,7 @@ adw_view_stack_set_hhomogeneous (AdwViewStack *self,
 }
 
 /**
- * adw_view_stack_get_vhomogeneous: (attributes org.gtk.Method.get_property=vhomogeneous)
+ * adw_view_stack_get_vhomogeneous:
  * @self: a view stack
  *
  * Gets whether @self is vertically homogeneous.
@@ -1959,7 +2217,7 @@ adw_view_stack_get_vhomogeneous (AdwViewStack *self)
 }
 
 /**
- * adw_view_stack_set_vhomogeneous: (attributes org.gtk.Method.set_property=vhomogeneous)
+ * adw_view_stack_set_vhomogeneous:
  * @self: a view stack
  * @vhomogeneous: whether to make @self vertically homogeneous
  *
@@ -1991,7 +2249,133 @@ adw_view_stack_set_vhomogeneous (AdwViewStack *self,
 }
 
 /**
- * adw_view_stack_get_pages: (attributes org.gtk.Method.get_property=pages)
+ * adw_view_stack_get_enable_transitions:
+ * @self: a view stack
+ *
+ * Gets whether @self uses a crossfade transition between pages.
+ *
+ * Use [property@ViewStack:transition-duration] to control the duration, and
+ * [property@ViewStack:transition-running] to know when the transition is
+ * running.
+ *
+ * Returns: whether to enable page transitions
+ *
+ * Since: 1.7
+ */
+gboolean
+adw_view_stack_get_enable_transitions (AdwViewStack *self)
+{
+  g_return_val_if_fail (ADW_IS_VIEW_STACK (self), FALSE);
+
+  return self->enable_transitions;
+}
+
+/**
+ * adw_view_stack_set_enable_transitions:
+ * @self: a view stack
+ * @enable_transitions: whether to enable page transitions
+ *
+ * Sets whether @self uses a crossfade transition between pages.
+ *
+ * Since: 1.7
+ */
+void
+adw_view_stack_set_enable_transitions (AdwViewStack *self,
+                                       gboolean      enable_transitions)
+{
+  g_return_if_fail (ADW_IS_VIEW_STACK (self));
+
+  enable_transitions = !!enable_transitions;
+
+  if (enable_transitions == self->enable_transitions)
+    return;
+
+  self->enable_transitions = enable_transitions;
+
+  if (self->enable_transitions) {
+    adw_timed_animation_set_duration (ADW_TIMED_ANIMATION (self->animation),
+                                      self->transition_duration);
+  } else {
+    adw_timed_animation_set_duration (ADW_TIMED_ANIMATION (self->animation), 0);
+  }
+
+  g_object_notify_by_pspec (G_OBJECT (self), props[PROP_ENABLE_TRANSITIONS]);
+}
+
+/**
+ * adw_view_stack_get_transition_duration:
+ * @self: a view stack
+ *
+ * Gets the transition animation duration for @self.
+ *
+ * Returns: the transition duration, in milliseconds
+ *
+ * Since: 1.7
+ */
+guint
+adw_view_stack_get_transition_duration (AdwViewStack *self)
+{
+  g_return_val_if_fail (ADW_IS_VIEW_STACK (self), 0);
+
+  return self->transition_duration;
+}
+
+/**
+ * adw_view_stack_set_transition_duration:
+ * @self: a view stack
+ * @duration: the new duration, in milliseconds
+ *
+ * Sets the transition animation duration for @self.
+ *
+ * Only used when [property@ViewStack:enable-transitions] is set to `TRUE`.
+ *
+ * Since: 1.7
+ */
+void
+adw_view_stack_set_transition_duration (AdwViewStack *self,
+                                        guint         duration)
+{
+  g_return_if_fail (ADW_IS_VIEW_STACK (self));
+
+  if (self->transition_duration == duration)
+    return;
+
+  self->transition_duration = duration;
+
+  if (self->enable_transitions) {
+    adw_timed_animation_set_duration (ADW_TIMED_ANIMATION (self->animation),
+                                      self->transition_duration);
+  } else {
+    adw_timed_animation_set_duration (ADW_TIMED_ANIMATION (self->animation), 0);
+  }
+
+  g_object_notify_by_pspec (G_OBJECT (self), props[PROP_TRANSITION_DURATION]);
+}
+
+/**
+ * adw_view_stack_get_transition_running:
+ * @self: a view stack
+ *
+ * Gets whether a transition is currently running for @self.
+ *
+ * If a transition is impossible, the property value will be set to `TRUE` and
+ * then immediately to `FALSE`, so it's possible to rely on its notifications
+ * to know that a transition has happened.
+ *
+ * Returns: whether a transition is currently running
+ *
+ * Since: 1.7
+ */
+gboolean
+adw_view_stack_get_transition_running (AdwViewStack *self)
+{
+  g_return_val_if_fail (ADW_IS_VIEW_STACK (self), FALSE);
+
+  return self->transition_running;
+}
+
+/**
+ * adw_view_stack_get_pages:
  * @self: a view stack
  *
  * Returns a [iface@Gio.ListModel] that contains the pages of the stack.
@@ -2010,8 +2394,8 @@ adw_view_stack_get_pages (AdwViewStack *self)
   if (self->pages)
     return g_object_ref (self->pages);
 
-  self->pages = GTK_SELECTION_MODEL (adw_view_stack_pages_new (self));
-  g_object_add_weak_pointer (G_OBJECT (self->pages), (gpointer *) &self->pages);
+  g_set_weak_pointer (&self->pages,
+                      GTK_SELECTION_MODEL (adw_view_stack_pages_new (self)));
 
   return self->pages;
 }
